from __future__ import print_function

import logging
import leip
import os
import re

from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)


@leip.hook("madfile_post_save", 100)
def root(app, madfile):

    if os.geteuid() != 0:
        #only do something when root
        return

    lg.debug("changing .madfile permission")

    filename = madfile.otf.filename
    madname = madfile.otf.madname

    if os.path.exists(madname):
        fstats = os.stat(filename)
        os.chmod(madname, fstats.st_mode)
        os.chown(madname, fstats.st_uid, fstats.st_gid)



from __future__ import print_function

import datetime
import logging
import sys
import os
import leip
import hashlib

from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)

def get_qdhash(filename):
    """
    Provde a quick & dirty hash - a good indication that a file MIGHT have
    changed - but by no means secure.

    It is quick, though.
    """
    sha1sum = hashlib.sha1()
    filesize = os.stat(filename).st_size
    if filesize < 20000:
        with open(filename, 'rb') as F:
            sha1sum.update(F.read().encode())
    else:
        with open(filename, 'rb') as F:
            for x in range(9):
                F.seek(int(filesize * (x/10.0)))
                sha1sum.update(F.read(2000))

            F.seek(-2000, 2)
            sha1sum.update(F.read())

    return sha1sum.hexdigest()


def get_mtime(fn):
    return datetime.datetime.utcfromtimestamp(
        os.stat(fn).st_mtime).isoformat()

@leip.hook("madfile_load", 150)
def hashhelper(app, madfile):
    """
    Calculate a quick&dirty checksum

    """

    may_have_changed = False
    if madfile.mad.hash.qdhash:
        qmt = madfile.mad.hash.mtime
        mtime =get_mtime(madfile.filename)
        if qmt != mtime:
            may_have_changed = True
    elif madfile.mad.hash.mtime:
        qdh = madfile.mad.hash.qdhash
        cs = get_qdhash(madfile.filename)
        if qdh != cs:
            may_have_changed = True

    if may_have_changed:
        print("{} may have changed! (rerun mad sha1)".format(madfile.filename),
              file=sys.stderr)

def hashit(hasher, filename):
    """
    Provde a quick & dirty hash

    this is by no means secure, but quick for very large files, and as long
    as one does not try to create duplicate hashes, the chance is still very
    slim that a duplicate will arise
    """
    h = hasher()
    blocksize = 2**20
    with open(filename, 'rb') as F:
        for chunk in iter(lambda: F.read(blocksize), b''):
            h.update(chunk)
    return h.hexdigest()


@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.command
def sha1(app, args):
    """
    Calculate a sha1 checksum
    """

    for madfile in get_all_mad_files(app, args):
        if not args.force and 'sha1' in madfile.mad:
            #exists - and not forcing
            lg.warning("Skipping sha1 checksum - exists")
            continue

        qd = get_qdhash(madfile.filename)
        mtime =get_mtime(madfile.filename)

        madfile.mad.hash.qdhash = qd
        madfile.mad.hash.mtime = mtime

        cs = hashit(hashlib.sha1, madfile.filename)
        madfile.mad.hash.sha1 = cs

        madfile.save()
        print(madfile.filename)





from __future__ import print_function


import logging
import leip
import os
import re

lg = logging.getLogger(__name__)
#lg.setLevel(logging.DEBUG)

from sqlalchemy.exc import OperationalError
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String

Base = declarative_base()

class SqlMadFile(Base):
    __tablename__ = 'madfile'

    id = Column(Integer, primary_key=True)

    basename = Column(String)
    dirname = Column(String)
    filesize = Column(Integer)
    host = Column(String)
    madname = Column(String)
    owner = Column(String)
    project = Column(String)
    userid = Column(String)
    username = Column(String)

    def __init__(self, maf):
        self.basename = maf.basename
        self.dirname = maf.dirname
        self.filesize = maf.otf.filesize
        self.host = maf.host
        self.madname = maf.madname
        self.owner = maf.owner
        self.project = maf.project
        self.userid = maf.userid
        self.username = maf.username

    def __repr__(self):
       return "<SqlMadFile('%s')>" % (self.basename)

def get_session(app):
    global Base
    enginestr = app.conf.plugin.sql.engine
    lg.warning("opening db @ {}".format(enginestr))
    engine = create_engine(enginestr)
    Session = sessionmaker(bind=engine)
    Base.metadata.create_all(engine)
    return Session()

def save_madfile(app, madfile):
    session = get_session(app)
    M = SqlMadFile(madfile)
    session.add(M)
    session.commit()

@leip.hook("madfile_save", 200)
def sqlsave(app, madfile):

    lg.critical("start save to %s" % app.conf.plugin.sql.engine)
    save_madfile(app, madfile)



from __future__ import print_function

import logging
import os
import sys

import leip

lg = logging.getLogger(__name__)

@leip.arg('-n', '--no_recurse', action='store_true', help='no recursive search')
@leip.command
def find(app, args):
    """
    (recursively) find and print files already mad annotated
    """
    for dirpath, dirnames, filenames in os.walk('.'):
        for f in filenames:
            if f[0] == '.' and f[-4:] == '.mad':
                print(os.path.join(dirpath, f[1:-4]))
        if dirpath == '.' and args.no_recurse:
            break


import logging
import re
import select
import sys

from mad2.exception import MadPermissionDenied, MadNotAFile
from mad2.madfile import MadFile

lg = logging.getLogger(__name__)

##
## Helper function - instantiate a madfile, and provide it with a
## method to run hooks
##
def get_mad_file(app, filename):
    """
    Instantiate a mad file & add hooks
    """
    lg.debug("instantiating madfile for {0}".format(filename))
    return MadFile(filename, hook_method = app.run_hook)

def get_filenames(args):
    """
    Get all incoming filenames
    """
    filenames = []
    demad = re.compile(r'^\.(.*)\.mad$')

    if 'file'in args and len(args.file) > 0:
        filenames.extend([demad.sub(r'\1', x)
                         for x in args.file])
    else:
        #nothing in args - see if there is something on stdin
        filenames.extend(
                [demad.sub(r'\1', x)
                 for x in sys.stdin.read().split("\n") if x != ''])

    filenames = sorted(list(set(filenames)))
    return filenames


def get_all_mad_files(app, args):
    """
    get input files from sys.stdin and args.file
    """
    for filename in get_filenames(args):
        try:
            yield get_mad_file(app, filename)
        except MadNotAFile:
            pass
        except MadPermissionDenied:
            lg.warning("Permission denied: {}".format(
                filename))


def render(txt, data):

    env = moa.moajinja.getStrictEnv()
    renconf = self.render()
    templ = env.from_string(value)
    try:
        rv = templ.render(renconf)
        return rv
    except jinja2.exceptions.UndefinedError:
        return value
    except jinja2.exceptions.TemplateSyntaxError:
        return value



from __future__ import print_function

import argparse
import logging
import re
import os
import sys

import leip
from mad2.util import  get_filenames, get_all_mad_files

lg = logging.getLogger(__name__)

@leip.arg('file', nargs='*')
@leip.arg('--dry', help = 'dry run', action='store_true')
@leip.command
def catchup(app, args):
    """
    execute all deferred commands
    """
    for madfile in get_all_mad_files(app, args):

        if madfile.mad.execute.queue:
            queue = madfile.execute.queue
            if not args.dry:
                del(madfile.execute.queue)
                madfile.save()

            executed = []
            for command in queue:
                if args.dry:
                    print(command)
                else:
                    cl = execute(app, madfile, command)
                    executed.append(cl)

            madfile.load()
            if not madfile.execute.history:
                madfile.execute.history = []
            madfile.execute.history.extend(executed)
            madfile.save()


def execute(app, madfile, cl, dry=False):
    """
    execute a command line in the context of this object

    :param dry: do a dry run
    :type dry: boolean
    """
    rendered = madfile.render(cl, app.conf)
    lg.info("executing: {0}".format(rendered))
    if dry:
        print(rendered)
    else:
        os.system(rendered)

    return rendered

def update_history(madfile, executed):
    """
    Add a list of commands to the history

    :param executed: a list of executed commands
    """
    madfile.load()
    if not madfile.execute.history:
        madfile.execute.history = []
    madfile.execute.history.extend(executed)
    madfile.save()


@leip.arg('file', nargs='*')
@leip.arg('comm', metavar='command', help='command to execute (use quotes!)')
@leip.arg('-s', '--save', action='store_true', help='save for later execution')
@leip.arg('--dry', help='dry run', action='store_true')
@leip.command
def x(app, args):
    """
    Execute a command
    """
    command = args.comm
    lg.info("command to execute: {0}".format(command))

    for madfile in get_all_mad_files(app, args):
        if args.save:
            if not madfile.mad.execute.queue:
                madfile.mad.execute.queue = []
            madfile.mad.execute.queue.append(command)
            madfile.save()
        else:
            cl = execute(app, madfile, command, dry=args.dry)



# -*- coding: utf-8 -*-
from __future__ import print_function,  unicode_literals

from dateutil.parser import parse as dateparse
import logging
import os
import sys
import subprocess
import tempfile

import leip

import mad2.ui

from mad2.util import  get_mad_file, get_all_mad_files

from signal import signal, SIGPIPE, SIG_DFL

#Ignore SIG_PIPE and don't throw exceptions
#otherwise it crashes when you pipe into, for example, a head
#see http://newbebweb.blogspot.be/2012/02/python-head-ioerror-errno-32-broken.html
#see http://docs.python.org/library/signal.html
signal(SIGPIPE, SIG_DFL)
lg = logging.getLogger(__name__)



def dispatch():
    """
    Run the app - this is the actual entry point
    """
    app.run()


##
## define Mad commands
##

@leip.arg('file', nargs='?')
@leip.arg('key', help='key to set')
@leip.command
def edit(app, args):
    """
    Edit a key in a full screen editor
    """
    key = args.key
    editor = os.environ.get('EDITOR','vim')

    if args.file:
        madfile = get_mad_file(app, args.file)
        default = madfile.mad.get(key, "")
    else:
        #if no file is defined, use the configuration
        default = app.conf.get(key, "")

    #write default value to a temp file, and start the editor
    tmp_file = tempfile.NamedTemporaryFile('wb', delete=False)
    if default:
        tmp_file.write(default)
    tmp_file.close()
    subprocess.call([editor, tmp_file.name])

    #read tmp file
    with open(tmp_file.name, 'r') as F:
        #removing trailing space
        val = F.read().rstrip()

    if args.file:
        madfile.mad[key] = val
        madfile.save()
    else:
        app.conf[key] = val
        app.conf.save()

    os.unlink(tmp_file.name)


@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.arg('value', help='value to set')
@leip.arg('key', help='key to set')
@leip.command
def set(app, args):
    """
    Set a key/value for one or more files.

    Use this command to set a key value pair for one or more files.

    This command can take the following forms:

        mad set project test genome.fasta
        ls *.fasta | mad set project test
        find . -size +10k | mad set project test
        mad set project - genome.fasta
        >
    """

    key = args.key
    val = args.value

    madfiles = list(get_all_mad_files(app, args))

    if val == '-':
        #Show a prompt asking for the question
        if len(madfiles) == 1:
            data = madfiles[0].data(app.conf)
            default = madfiles[0].mad.get(key, "")
        else:
            data = app.conf.simple()

        val = mad2.ui.askUser(key, default, data)


    lg.debug("processing %d files" % len(madfiles))

    #if len(madfiles) == 0:
    #    #apply conf to the local user config if no madfiles are defined
    #    app.conf[key] = val
    #    app.conf.save()
    #    return

    keywords = app.conf.keywords
    if not args.force and not key in keywords:
        print("invalid key: {0} (use -f?)".format(key))
        sys.exit(-1)

    keyinfo = keywords[key]
    keytype = keyinfo.get('type', 'str')

    if keytype == 'int':
        try:
            val = int(val)
        except ValueError:
            lg.error("Invalid integer: %s" % val)
            sys.exit(-1)
    elif keytype == 'float':
        try:
            val = float(val)
        except ValueError:
            lg.error("Invalid float: %s" % val)
            sys.exit(-1)
    elif keytype == 'boolean':
        if val.lower() in ['1', 'true', 't', 'yes', 'aye', 'y', 'yep']:
            val = True
        elif val.lower() in ['0', 'false', 'f', 'no', 'n', 'nope']:
            val = False
        else:
            lg.error("Invalid boolean: %s" % val)
            sys.exit(-1)
    elif keytype == 'date':
        try:
            val = dateparse(val)
        except ValueError:
            lg.error("Invalid date: %s" % val)
            sys.exit(-1)
        lg.warning("date interpreted as: %s" % val)


    for madfile in madfiles:
        list_mode = False
        if key[0] == '+':
            list_mode = True
            key = key[1:]



        if list_mode and keyinfo.cardinality == '1':
            print("Cardinality == 1 - no lists!")
            sys.exit(-1)
        elif keyinfo.cardinality == '+':
            list_mode = True

        if keytype == 'restricted' and \
                not val in keyinfo.allowed:
            print("Value '{0}' not allowed".format(val))
            sys.exit(-1)

        if list_mode:
            if not key in madfile.mad:
                oldval = []
            else:
                oldval = madfile.mad[key]
                if not isinstance(oldval, list):
                    oldval = [oldval]
            madfile.mad[key] = oldval + [val]
        else:
            #not listmode
            madfile.mad[key] = val

        madfile.save()



##
## define show
##
@leip.arg('-a', '--all', action='store_true')
@leip.arg('file', nargs='*')
@leip.command
def show(app, args):
    i = 0
    for madfile in get_all_mad_files(app, args):
        d = madfile.mad.copy()
        d.update(madfile.otf)
        if i > 0:
            print('---')
        print(d.pretty().strip())
        i += 1


##
## define show
##
@leip.arg('-a', '--all', action='store_true')
@leip.arg('file', nargs='*')
@leip.command
def show(app, args):
    i = 0
    for madfile in get_all_mad_files(app, args):
        d = madfile.mad.copy()
        d.update(madfile.otf)
        if i > 0:
            print('---')
        print(d.pretty().strip())
        i += 1

##
## define show
##
@leip.arg('file', nargs="+")
@leip.arg('key')
@leip.command
def unset(app, args):
    lg.debug("unsetting: %s".format(args.key))
    key = args.key
    keywords = app.conf.keywords
    keyinfo = keywords[key]
    if keyinfo.cardinality =='+':
        print("Not implemented - unsetting keys with cardinality > 1")
        sys.exit(-1)

    for madfile in get_all_mad_files(app, args):
        if args.key in madfile.mad:
            del(madfile.mad[args.key])
        madfile.save()


@leip.commandName('key')
def madkeylist(app, args):
    kyw = sorted(app.conf.keywords)
    mxlen = str(max([len(x) for x in kyw])+1)
    for c in sorted(app.conf.keywords):
        print(("{:<" + mxlen + "} {}").format(c, app.conf.keywords[c].description))


@leip.commandName('category')
def madcatlist(app, args):
    kyw = sorted(app.conf.keywords.category.allowed)
    mxlen = str(max([len(x) for x in kyw])+1)
    for c in sorted(kyw):
        dsc = app.conf.keywords.category.allowed[c]
        print(("{:<" + mxlen + "} {}").format(c, dsc))
        tmpl = app.conf.template[c]
        if not tmpl: continue
        mxlen2 = str(max([len(x) for x in tmpl])+1)
        for k in sorted(tmpl):
            print(("  - {:<" + mxlen2 + "} {}").format(k, app.conf.template[c][k]))



@leip.arg('comm', metavar='command', help='command to check')
@leip.command
def has_command(app, args):
    comm = args.comm
    lg.info("checking if we know command %s" % comm)
    if comm in app.leip_commands:
        sys.exit(0)
    else:
        sys.exit(-1)

##
## Instantiate the app and discover hooks & commands
##

#trail of config files???
config_files = [
    '/etc/mad2.config',
    '~/.config/mad/mad2.config']
path = os.getcwd()
config_no = 0
xtra_config = []
while path:
    config_no += 1
    xtra_config.append(os.path.join(path, '.mad'))
    path = path.rsplit(os.sep, 1)[0]

config_files.extend(list(reversed(xtra_config)))
#print(config_file)
app = leip.app(name='mad2', set_name=None,
               config_files = config_files)

#discover hooks in this module!
app.discover(globals())



from __future__ import print_function

import logging
import sys

import leip
from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)


@leip.arg('file', nargs='*')
@leip.arg('category', help='category to assign to these files')
@leip.commandName('=')
def apply_category(app, args):
    """
    apply a predefined category to a (set of) file(s)
    """
    if not args.category in app.conf.template:
        print("Invalid category: {0}".format(args.category))
        print("Choose from:")
        for cat in app.conf.template:
            print(" - {0}".format(cat))
        sys.exit()
    template_data = app.conf.template[args.category]
    for madfile in get_all_mad_files(app, args):
        madfile.mad.update(template_data)
        madfile.save()



from __future__ import print_function

from datetime import datetime
import os
import logging
import socket

import leip
from pwd import getpwuid

lg = logging.getLogger(__name__)

EXTENSION_DATA = None


def get_fiex(app):
    global EXTENSION_DATA
    if not EXTENSION_DATA is None:
        return EXTENSION_DATA

    EXTENSION_DATA = {}
    for ft in app.conf.filetype:
        for ext in app.conf.filetype.get(ft).extensions:
            EXTENSION_DATA[ext] = ft

    return EXTENSION_DATA


def apply_file_format(app, madfile, filename=None):

    extension_data = get_fiex(app)

    if filename is None:
        filename = madfile.otf.basename

    splitter = filename.rsplit('.', 1)
    if len(splitter) != 2:
        return

    base, ext = splitter

    #this ensures that the innermost extension seen is stored
    madfile.otf.extension = ext

    if not ext in extension_data:
        return

    ft = extension_data[ext]
    ftinfo = app.conf.filetype.get(ft)

    lg.debug("identified filetype %s" % ft)
    if ftinfo.template:
        template = app.conf.template.get(ftinfo.template)
        madfile.otf.update(template)

    if ftinfo.get('continue', False):
        lg.debug("contiue filetype disocvery on: %s" % base)
        apply_file_format(app, madfile, base)


@leip.hook("madfile_load")
def onthefly(app, madfile):

    if sorted(list(madfile.mad.keys())) == ['hash']:
        madfile.otf.annotated = False
    else:
        madfile.otf.annotated = True

    lg.debug("running onthelfy")
    madfile.otf.fullpath = os.path.abspath(madfile.filename)
    lg.debug("get fqdn")
    madfile.otf.host = socket.gethostname()

    filestat = os.stat(madfile.filename)
    madfile.otf.filesize = filestat.st_size
    userinfo = getpwuid(filestat.st_uid)
    madfile.otf.userid = userinfo.pw_name
    madfile.otf.username = userinfo.pw_gecos

    mtime = datetime.utcfromtimestamp(
        filestat.st_mtime)
    atime = datetime.utcfromtimestamp(
        filestat.st_atime)

    madfile.otf.atime = atime.isoformat()
    madfile.otf.atime_simple = atime.strftime("%m%y")

    madfile.otf.mtime = mtime.isoformat()
    madfile.otf.mtime_simple = mtime.strftime("%m%y")

    apply_file_format(app, madfile)

    lg.debug("finished onthefly")



from __future__ import print_function

import re
import logging
import sys

import leip
from mad2.util import  get_filenames, get_all_mad_files

lg = logging.getLogger(__name__)

OPERATORS =  {'<' : lambda x,y: x < y,
              '<=' : lambda x,y: x <= y,
              '==' : lambda x,y: x == y,
              '=' : lambda x,y: x == y,
              '' : lambda x,y: x == y,
              '>' : lambda x,y: x > y,
              '>=' : lambda x,y: x > y,
              '!=' : lambda x,y: x != y,
              '!' : lambda x,y: x != y,
              '<>' : lambda x,y: x != y
              }


def is_numeric(val):
    if numre:
        return True
    return False
    
@leip.arg('file', nargs='*')
@leip.arg('value', help='value to filter with')
@leip.arg('key', help='key to filter on')
@leip.command
def select(app, args):
    """
    Select a set of files based on their metadata
    """
    val = args.value
    lg.debug("compare {0} vs '{1}'".format(args.key, val))

    def compare_func(what):
        return what == val
    
    lg.debug("compare {0} vs '{1}'".format(args.key, val))
    #see if it is a numeric or regex comparison:
    numre = re.match('([><!=]{,2})([0-9\.]{1,})', val)
    if numre:
        operator, number = numre.groups()
        def compare_func(a):
            cf =  OPERATORS.get(operator, lambda x,y: x==y)
            return cf(float(a), float(number))
    elif val[0] == '/' and val[-1] == '/':
        regex = re.compile(val[1:-1])
        def compare_func(a):
            return regex.search(a)

    for madfile in get_all_mad_files(app, args):
        kv = madfile.mad.get(args.key, None)
        if not kv is None and compare_func(kv):
            print(madfile.filename)

                
    
    


            



from __future__ import print_function

import logging
import sys
import re
import xlrd

import leip
from mad2.util import  get_filenames, get_all_mad_files

lg = logging.getLogger(__name__)
    

@leip.arg('file', nargs='*')
@leip.arg('samplesheet', help='sheet with information')
@leip.arg('--id', help='header of identifying column')
@leip.arg('--format', help='format string to identify file', 
          default='{0}')
@leip.arg('--sanitize', help='sanitize by removing special characters', 
          action='store_true')
@leip.arg('--multiple', help='multiple files per id allowed', 
          action='store_true')
@leip.arg('--apply', action='store_true', help='actually apply')
@leip.command
def samplesheet(app, args):
    """
    Apply samplesheet data
    """
    wb = xlrd.open_workbook(args.samplesheet)
    lg.debug("discovered sheets: %s" % ', '.join(wb.sheet_names()))
    sheetName = wb.sheet_names()[0]
    lg.debug("using sheet: '%s'" % sheetName)
    sheet = wb.sheet_by_name(sheetName)
    header = sheet.row_values(0)
    resanitize = re.compile(r'[^A-Za-z0-9]')
    def show_columns():
        print("Select from:")
        for col in header:
            print('- {0}'.format(col))
    
    if not args.id:
        print("Need a column with identifying information (--id)")
        show_columns()
        sys.exit(-1)

    if args.id in header:
        id_values = sheet.col_values(header.index(args.id), start_rowx=1)
        if args.sanitize:
            id_values = [resanitize.sub('', x) for x in id_values]
    else:
        print("Unknown column with identifying information (--id {0})".format(args.id))
        show_columns()
        sys.exit(-1)


    if len(id_values) != len(set(id_values)):
        print("duplicate ids")
        sys.exit(-1)

    lg.debug("id column: {0}".format(args.id))

    filenames = sorted(list(get_filenames(args)))

    #try to map values to files
    file2id = {}
    id2file = {}
    for fid in id_values:
        for filename in filenames:
                
            if args.sanitize:
                san_filename = resanitize.sub('', filename)
            else:
                san_filename = filename

            if args.format.format(fid) in san_filename:
                if filename in file2id:
                    print("filename to id clash: {0} - {1}/{2}".format(
                            fid, filename, file2id[filename]))
                    sys.exit(-1)
                if fid in id2file:
                    print("id to filename clash: {0} - {1}/{2}".format(
                            filename, fid, id2file[fid]))
                    sys.exit(-1)

                file2id[filename] = fid
                id2file[fid] = filename

    no_apply = 0
    if not args.apply:
        print("found mapping (apply with --apply):")
        for filename in filenames:
            fid = file2id.get(filename, '<none>')
            if fid != '<none>':
                no_apply += 1
            print(' - {0} : {1}'.format(fid, filename))
        if no_apply == 0:
            print("none applied - first few id values are:")
            print(", ".join(id_values[:5]))
        sys.exit()
    
    for madfile in get_all_mad_files(app, args):
        fid = file2id.get(madfile.filename)
        if fid in id_values:
            rowid = id_values.index(fid) + 1
            data = dict(zip(header, sheet.row_values(rowid)))
            madfile.mad.update(data)
            madfile.save()

                
    
    


            



from __future__ import print_function

import logging
from pymongo import MongoClient
from bson.objectid import ObjectId
import leip
import copy
import datetime
import pprint
from mad2.util import get_all_mad_files


lg = logging.getLogger(__name__)
#lg.setLevel(logging.DEBUG)

def mongo_prep_mad(mf):
    d = mf.otf.copy()
    d.update(mf.mad)
    d.sha1 = d.hash.sha1
    d.mtime = d.hash.mtime
    del d.mongo
    del d.hash
    return d

@leip.hook("madfile_save")
def store_in_mongodb(app, madfile):
    lg.debug("running store_in_mongodb")
    mng_mad = _get_mng_collection(app)
    if madfile.mad.mongo.id:
        mongoid = madfile.mad.mongo.id
        newrec = mongo_prep_mad(madfile)
        newrec['_id'] = ObjectId(mongoid)
        mng_mad.save(newrec)
    else:
        newrec = mongo_prep_mad(madfile)
        mongoid = mng_mad.insert(newrec)
        lg.debug("Updating mongo record {0}".format(mongoid))
        madfile.mad.mongo.id = str(mongoid)

def _get_mng_collection(app):
    host = app.conf.plugin.mongo.host
    port = app.conf.plugin.mongo.get('port', 27017)
    client = MongoClient(host, port)
    return client.mad.mad

@leip.arg('file', nargs="+")
@leip.command
def mongo_show(app, args):
    """
    Show the associated mongodb record
    """
    mng_mad = _get_mng_collection(app)
    for madfile in get_all_mad_files(app, args):
        if madfile.mad.mongo.id:
            print ('---', madfile.filename)
            mongoid = madfile.mad.mongo.id
            rec = mng_mad.find_one({'_id': ObjectId(mongoid)})
            #print(madfile.filename)
            if not rec:
                continue
            for key in rec:
                if key == 'mongo':
                    continue
                print('- {0}: {1}'.format(key, rec[key]))

@leip.command
def mongo_drop(app, args):
    """
    Show the associated mongodb record
    """
    mng_mad = _get_mng_collection(app)
    mng_mad.drop()

@leip.command
def mongo_index(app, args):
    """
    Ensure indexes on the relevant fields
    """
    mng_mad = _get_mng_collection(app)
    for f in app.conf.plugin.mongo.indici:
        mng_mad.ensure_index(f)






from __future__ import print_function

import logging
import leip
import os
import re

from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)

@leip.hook("madfile_load", 100)
def lbconthefly(app, madfile):

    fullpath = madfile.otf.fullpath
    mtcher = re.compile(r'/media/(seq-srv-[0-9][0-9])', re.I)
    mtch = mtcher.search(fullpath)
    if not mtch:
        return


    server = mtch.groups()[0]

    madfile.otf.host = server.upper()
    fp = madfile.otf.fullpath
    fp = fp[fp.index(server) + len(server):]
    dn = os.path.dirname(fp)
    madfile.otf.fullpath = fp
    madfile.otf.dirname = dn
